@@ -19,10 +19,10 @@ import mock
 
 from lazysignup.backends import LazySignupBackend
 from lazysignup.decorators import allow_lazy_user
+from lazysignup.exceptions import NotLazyError
 from lazysignup.management.commands import remove_expired_users
-
-from lazysignup.utils import username_from_session
-from lazysignup.templatetags.lazysignup_tags import is_lazy_user
+from lazysignup.models import LazyUser
+from lazysignup.utils import is_lazy_user, username_from_session
 
 class GoodUserCreationForm(UserCreationForm):
     """ Hardcoded credentials to demonstrate that the get_credentials method
@@ -50,7 +50,7 @@ def view(request):
     if request.user.is_authenticated():
         r.status_code = 500
     return r
-    
+
 def lazy_view(request):
     from django.http import HttpResponse
     r = HttpResponse()
@@ -76,16 +76,16 @@ def no_lazysignup(func):
                 settings.LAZYSIGNUP_ENABLE = old
         return result
     return wraps(func)(wrapped)
-        
+
 
 class LazyTestCase(TestCase):
 
     urls = 'lazysignup.test_urls'
-    
+
     def setUp(self):
         self.request = HttpRequest()
         SessionMiddleware().process_request(self.request)
-    
+
     @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
     def testSessionAlreadyExists(self, mock_resolve):
         # If the user id is already in the session, this decorator should do nothing.
@@ -94,7 +94,7 @@ class LazyTestCase(TestCase):
         self.request.user = AnonymousUser()
         login(self.request, authenticate(username='test', password='test'))
         mock_resolve.return_value = (f, None, None)
-        
+
         f(self.request)
         self.assertEqual(user, self.request.user)
 
@@ -105,11 +105,11 @@ class LazyTestCase(TestCase):
         f = allow_lazy_user(lambda request: 1)
         self.request.session[SESSION_KEY] = 1000
         mock_resolve.return_value = (f, None, None)
-        
+
         f(self.request)
         self.assertEqual(username_from_session(self.request.session.session_key), self.request.user.username)
         self.assertEqual(False, self.request.user.has_usable_password())
-        
+
     @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
     def testCreateLazyUser(self, mock_resolve):
         # If there isn't a setup session, then this middleware should create a user
@@ -119,7 +119,7 @@ class LazyTestCase(TestCase):
         f(self.request)
         self.assertEqual(username_from_session(self.request.session.session_key), self.request.user.username)
         self.assertEqual(False, self.request.user.has_usable_password())
-        
+
     @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
     def testBannedUserAgents(self, mock_resolve):
         # If the client's user agent matches a regex in the banned
@@ -127,11 +127,11 @@ class LazyTestCase(TestCase):
         self.request.META['HTTP_USER_AGENT'] = 'search engine'
         f = allow_lazy_user(lambda request: 1)
         mock_resolve.return_value = (f, None, None)
-        
+
         f(self.request)
         self.failIf(hasattr(self.request, 'user'))
         self.assertEqual(0, len(User.objects.all()))
-        
+
     def testNormalView(self):
         # Calling our undecorated view should *not* create a user. If one is created, then the
         # view will set the status code to 500.
@@ -145,7 +145,7 @@ class LazyTestCase(TestCase):
         response = self.client.get('/lazy/')
         self.assertEqual(200, response.status_code)
         self.assertEqual(1, len(User.objects.all()))
-        
+
     def testRemoveExpiredUsers(self):
         # Users wihout usable passwords who don't have a current session record should be removed.
         u1 = User.objects.create_user(username_from_session('dummy'), '')
@@ -156,14 +156,14 @@ class LazyTestCase(TestCase):
             expire_date=datetime.datetime.now() + datetime.timedelta(1)
         )
         s.save()
-        
+
         c = remove_expired_users.Command()
         c.handle()
-        
+
         users = User.objects.all()
         self.assertEqual(1, len(users))
         self.assertEqual(u1, users[0])
-        
+
     def testConvertAjax(self):
         # Calling convert with an AJAX request should result in a 200
         self.client.get('/lazy/')
@@ -173,16 +173,16 @@ class LazyTestCase(TestCase):
             'password2': 'password',
         }, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
         self.assertEqual(200, response.status_code)
-        
+
         users = User.objects.all()
         self.assertEqual(1, len(users))
         self.assertEqual('demo', users[0].username)
 
-        # We should find that the auth backend used is no longer the 
-        # Lazy backend, as the conversion should have logged the new 
+        # We should find that the auth backend used is no longer the
+        # Lazy backend, as the conversion should have logged the new
         # user in.
         self.assertNotEqual('lazysignup.backends.LazySignupBackend', self.client.session['_auth_user_backend'])
-        
+
     def testConvertNonAjax(self):
         # If it's a regular web browser, we should get a 301.
         self.client.get('/lazy/')
@@ -192,7 +192,7 @@ class LazyTestCase(TestCase):
             'password2': 'password',
         })
         self.assertEqual(302, response.status_code)
-        
+
         users = User.objects.all()
         self.assertEqual(1, len(users))
         self.assertEqual('demo', users[0].username)
@@ -220,7 +220,7 @@ class LazyTestCase(TestCase):
         }, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
         self.assertEqual(400, response.status_code)
         self.failIf(response.content.find('username') == -1)
-        
+
     def testConvertMismatchedNoAjax(self):
         self.client.get('/lazy/')
         response = self.client.post('/convert/', {
@@ -269,7 +269,7 @@ class LazyTestCase(TestCase):
         self.client.get('/lazy/')
         response = self.client.get('/convert/')
         self.assertEqual(200, response.status_code)
-        
+
     @no_lazysignup
     def testConvertAnon(self):
         # If the Convert view gets an anonymous user, it should redirect
@@ -277,7 +277,7 @@ class LazyTestCase(TestCase):
         response = self.client.get('/convert/')
         self.assertEqual(302, response.status_code)
         self.assertEqual('http://testserver' + settings.LOGIN_URL, response['location'])
-        
+
     def testConversionKeepsSameUser(self):
         self.client.get('/lazy/')
         response = self.client.post('/convert/', {
@@ -286,12 +286,12 @@ class LazyTestCase(TestCase):
             'password2': 'password',
         })
         self.assertEqual(1, len(User.objects.all()))
-        
+
     @no_lazysignup
     def testNoLazysignupDecorator(self):
         response = self.client.get('/lazy/')
         self.assertEqual(500, response.status_code)
-        
+
     def testBadCustomConvertForm(self):
         # Passing a form class to the conversion view that doesn't have
         # a get_credentials method should raise an AttributeError
@@ -310,10 +310,14 @@ class LazyTestCase(TestCase):
         })
         users = User.objects.all()
         self.assertEqual(1, len(users))
-        
+        user = users[0]
+
         # The credentials returned by get_credentials should have been used
-        self.assertEqual(users[0], authenticate(username='demo', password='demo'))
-        
+        self.assertEqual(user, authenticate(username='demo', password='demo'))
+
+        # The user should no longer be lazy
+        self.assertFalse(is_lazy_user(user))
+
     def testUsernameNotBasedOnSessionKey(self):
         # The generated username should not look like the session key. While doing
         # so isn't a security problem in itself, any client software that blindly
@@ -322,17 +326,17 @@ class LazyTestCase(TestCase):
         fake_session_key = 'a' * 32
         username = username_from_session(fake_session_key)
         self.failIf(fake_session_key.startswith(username))
-        
+
     def testDecoratorOrder(self):
         # It used to be the case that allow_lazy_user had to be first in the
         # decorator list. This is no longer the case.
         self.request.user = AnonymousUser()
         self.request.method = 'POST'
         v = require_POST(lazy_view)
-        
+
         response = v(self.request)
         self.assertEqual(200, response.status_code)
-        
+
     def testIsLazyUserAnonymous(self):
         user = AnonymousUser()
         self.assertEqual(False, is_lazy_user(user))
@@ -349,7 +353,17 @@ class LazyTestCase(TestCase):
         self.request.user = AnonymousUser()
         response = lazy_view(self.request)
         self.assertEqual(True, is_lazy_user(self.request.user))
-        
+
+    def testLazyUserNotLoggedIn(self):
+        # Check that the is_lazy_user works for users who were created
+        # lazily but are not the current logged-in user
+        user = LazyUser.objects.create_lazy_user('foo')
+        self.assertTrue(is_lazy_user(user))
+
+    def testAnonymousNotLazy(self):
+        # Anonymous users are not lazy
+        self.assertFalse(is_lazy_user(AnonymousUser()))
+
     def testBackendGetUserAnnotates(self):
         # Check that the lazysignup backend annotates the user object
         # with the backend, mirroring what Django's does
@@ -357,7 +371,27 @@ class LazyTestCase(TestCase):
         backend = LazySignupBackend()
         pk = User.objects.all()[0].pk
         self.assertEqual('lazysignup.backends.LazySignupBackend', backend.get_user(pk).backend)
-        
-        
-    
-    
+
+    def testConvertGood(self):
+        # Check that the convert() method on the lazy user manager
+        # correctly converts the lazy user
+        user = LazyUser.objects.create_lazy_user('foo')
+        d = {
+            'username': 'test',
+            'password1': 'password',
+            'password2': 'password',
+        }
+        form = GoodUserCreationForm(d, instance=user)
+        self.assertTrue(form.is_valid())
+
+        user = LazyUser.objects.convert(form)
+        self.assertFalse(is_lazy_user(user))
+
+    def testConvertNonLazy(self):
+        # Attempting to convert a non-lazy user should raise a TypeError
+        user = User.objects.create_user('dummy', 'dummy@dummy.com', 'dummy')
+        form = GoodUserCreationForm(instance=user)
+        self.assertRaises(NotLazyError, LazyUser.objects.convert, form)
+
+
+
